#include <windows.h>
#include "snes9x/snes9x.h"
#include "gui.h"

#ifdef UNDER_CE

#include "mapledev.h"
#include "Perstore.h"

static IFlashDevice *pIFlash[4][2] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
static int FlashFree[4][2] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static IFlashFile *pIFile = NULL;
static int sz;
int primary, port;

int GetSRAMSize(void) { return (sz / 512) + 1; }

int GetFlashStatus(int port, int dev) { 
	if(pIFlash[port][dev]) return FlashFree[port][dev];
	else return -1;
}

void GetCurrentFlashPort(int* p, int* dev) {
	*p = primary;
	*dev = port;
}

void S9xSaveSRAM(const char *filename, int size, const char *data) {
	char str[256];
	HRESULT hr;

	S9xMessage(0, 0, "S9xSaveSRAM called");
	sprintf(str, "Saving VMS File: %s", filename);
	S9xMessage(0, 0, str);

	// If we don't have a file already open, don't save...
	if(pIFlash == NULL) return;

	hr = pIFile->Write(0, size, (BYTE *)data);
	if(FAILED(hr)) {
		S9xMessage(0, 0, "IFlashFile::Write failed");
		return;
	}
	hr = pIFile->Flush();
	if(FAILED(hr)) {
		S9xMessage(0, 0, "IFlashFile::Flush failed");
		return;
	}

	
	S9xMessage(0, 0, "Save OK");
}

BOOL S9xLoadSRAM(const char *filename, int size, char* data) {
	char str[256];
	HRESULT hr;

	S9xMessage(0, 0, "S9xLoadSRAM called");
	sprintf(str, "Opening VMS File: %s", filename);
	S9xMessage(0, 0, str);
	
	hr = pIFlash[primary][port]->OpenFlashFileByName(&pIFile, (char *)filename);
	switch(hr) {
		case VMS_OK:
			S9xMessage(0, 0, "VMS File found.");
			hr = pIFile->Read(0, size, (BYTE *)data);
			if(FAILED(hr)) {
				S9xMessage(0, 0, "IFlashFile::Read failed");
				return TRUE;
			}
			S9xMessage(0, 0, "Load OK");
			return TRUE;
		case E_INVALIDARG:
			S9xMessage(0, 0, "Unexpected Error: Filename = NULL");
			break;
		case E_ACCESSDENIED:
			S9xMessage(0, 0, "File not Found");
			break;
		case E_OUTOFMEMORY:
			S9xMessage(0, 0, "Out of Memory");
			break;
		default:
			S9xMessage(0, 0, "General Error");
	}

	sz = size;
	return FALSE;
}

BOOL FlashCreateFile(const char *filename, const char *desc, int size) {
	char str[256];
	FSFILEDESC fsfd;

	memset(&fsfd, 0, sizeof(fsfd));

	fsfd.dwSize = sizeof(FSFILEDESC);
	fsfd.dwFlags = FSFD_CREATE_FILE;
	fsfd.dwBytesRequired = sz;
	strcpy(fsfd.szFileName, filename);
	strcpy(fsfd.szVMSComment, desc);
	strcpy(fsfd.szGameName, "Sintendo");
	strcpy(fsfd.szBootROMComment, "Sintendo");
	fsfd.bStatus = FS_STATUS_DATA_FILE;
	fsfd.bCopy = FS_COPY_ENABLED;

	S9xMessage(0, 0, "Creating file...");

	HRESULT hr = pIFlash[primary][port]->CreateFlashFile(&pIFile, &fsfd);
	if(FAILED(hr)) {
		sprintf(str, "HRESULT = %i %X", hr, hr);
		S9xMessage(0, 0, "CreateFlashFile failed");
		S9xMessage(0, 0, str);
		return FALSE;
	}

	S9xMessage(0, 0, "CreateFlashFile OK");

	return TRUE;
}

BOOL PASCAL EnumMapleDevicesCallback(LPCMAPLEDEVICEINSTANCE pmdi,
									 LPVOID context) {

	HRESULT hr;
	IUnknown *pIUnknown = NULL;
	IFlashDevice* pIFlashTemp = NULL;

	hr = MapleCreateDevice(&pmdi->guidDevice, &pIUnknown);
	if(FAILED(hr)) {
		S9xMessage(0, 0, "MapleCreateDevice Failed.");
		return TRUE;
	}

	hr = pIUnknown->QueryInterface(_uuidof(IFlashDevice), (void**)&pIFlashTemp);
	pIUnknown->Release();
	if(FAILED(hr)) {
		S9xMessage(0, 0, "QueryInterface failed.");
		return TRUE;
	}

//#ifdef DEBUG_FLASH2
//	S9xMessage(0, 0, "IFlashDevice found");
//#endif

	// Begin Query Flash Device	
	FSDEVICEDESC fsdevicedesc;

	memset(&fsdevicedesc, 0, sizeof(fsdevicedesc));
	fsdevicedesc.dwSize = sizeof(FSDEVICEDESC);
	fsdevicedesc.dwFlags = FSDD_PORT_NUMBER | FSDD_DEVICE_NUMBER | FSDD_FREE_BLOCKS;
	hr = pIFlashTemp->GetDeviceDesc(&fsdevicedesc);

	if(FAILED(hr)) {
		S9xMessage(0, 0, "GetDeviceDesc failed");
		return TRUE;
	}

//#ifdef DEGUB_FLASH2
//	char str[256];
//	sprintf(str, "FlashDevice Info: PORT: %i DEVICE: %i", fsdevicedesc.dwPortNumber,
//		fsdevicedesc.dwDeviceNumber);
//	S9xMessage(0, 0, str);
//	sprintf(str, "%i Free Blocks", fsdevicedesc.dwFreeBlocks);
//	S9xMessage(0, 0, str);
//#endif

	pIFlash[fsdevicedesc.dwPortNumber][fsdevicedesc.dwDeviceNumber-1] = pIFlashTemp;
	FlashFree[fsdevicedesc.dwPortNumber][fsdevicedesc.dwDeviceNumber-1] = fsdevicedesc.dwFreeBlocks;
	primary = fsdevicedesc.dwPortNumber;
	port = fsdevicedesc.dwDeviceNumber-1;

	return TRUE;
}

#endif

void FlashInit(void) {
#ifdef UNDER_CE
	HRESULT hr;

	hr = MapleEnumerateDevices(MDT_STORAGE, EnumMapleDevicesCallback, NULL, 0);

	if(FAILED(hr)) {
		S9xMessage(0, 0, "MapleEnumerateDevices failed");
	}
#endif
}

void FlashDeInit(void) {
#ifdef UNDER_CE
	if(pIFile) {
		pIFile->Flush();
		pIFile->Release();
		pIFile = NULL;
	}

	for(int i = 0; i < 4; i++) {
		for(int j = 0; j < 2; j++) {
			if(pIFlash[i][j] != NULL) {
				pIFlash[i][j]->Release();
				pIFlash[i][j] = NULL;
				FlashFree[i][j] = -1;
			}
		}
	}
#endif
}

#ifndef UNDER_CE

void S9xSaveSRAM(const char *filename, int size, const char *data) { return; }
BOOL S9xLoadSRAM(const char *filename, int size, char* data) { return FALSE; }
BOOL FlashCreateFile(const char *filename, const char *desc, int size) { return FALSE; }
int GetFlashStatus(int port, int dev) { 
	if(port == 0 && dev == 1) return 15;
	if(port == 2 && dev == 0) return 87;
	return -1;
}
void GetCurrentFlashPort(int* p, int* dev) { *p = 0, *dev = 0; }
int GetSRAMSize(void) { return 0; }

#endif