#include "gui.h"			// To avoid improper function declarations
#include "global.h"
#include "resource.h"
#include <ddraw.h>
#include <Winbase.h>
#include <d3d.h>
#include "input.h"
#include "emu.h"
#include <tchar.h>
#include "snes9x/memmap.h"
#include "bitmap.h"
#include "Direct3D.h"
#include "DCFlash.h"

#include "snes9x/snes9x.h"
#include "snes9x/cpuexec.h"

#define D3D_OVERLOADS
#define DO_SAVING

#define SPLASH_WAIT		1000	// Splash screen wait time, in milliseconds

#if defined MENU_640_480
	#define SCREENX			640
	#define SCREENY			480
	#define MENU_BORDER		64
	#define FONT_SIZE		32
	#define SMALL_FONT_SIZE	24
	#define ICONX			40
	#define ICONY			30
	#define LEFTOF_ICON		6
	#define LEFTOF_TEXT		8
	#define MEMORY_CARD_TOP_BOTTOM				64
	#define MEMORY_CARD_HEIGHT					120
	#define MEMORY_CARD_UP_DOWN_SEPERATOR		24
	#define MEMORY_CARD_LEFT_RIGHT				24
	#define MEMORY_CARD_WIDTH					98
	#define MEMORY_CARD_LEFT_RIGHT_SEPERATOR	24
	
#else
	#define SCREENX			320
	#define SCREENY			240
	#define MENU_BORDER		32
	#define FONT_SIZE		16
	#define SMALL_FONT_SIZE	12
	#define ICONX			19
	#define ICONY			15
	#define LEFTOF_ICON		3
	#define LEFTOF_TEXT		4
#endif
bool DoMenu(bool allowcancel = false);
void DoSaveLoadMenu(bool);
void DoFrameSkipMenu(void);

int gint_skipframes = 0;
int gint_skipoutof = 0;
int skipmenu = 0;


static LPDIRECTDRAWSURFACE4 splash = NULL;


// Global variables, usually gotten once and then left set, such as fonts or video buffers
// of particular use

static HFONT ghFontLarge = NULL, ghFontSmall = NULL;
static LPDIRECTDRAWSURFACE4 gSurfAccept = NULL, gSurfCancel = NULL, gSurfEnd = NULL;

void SetupGlobal(void) {
	LOGFONT lf;
	if(!ghFontLarge) {
		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = 32;
		lf.lfWeight = FW_BOLD;
		ghFontLarge = CreateFontIndirect(&lf);

		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = 24;
		//lf.lfWeight = FW_BOLD;
		ghFontSmall = CreateFontIndirect(&lf);
	}

	if(!gSurfAccept) {
		gSurfAccept = LoadBitmap(IDB_ACCEPT);
		gSurfCancel = LoadBitmap(IDB_CANCEL);
		gSurfEnd = LoadBitmap(IDB_END);
	}
}




extern void DoGUI(void) {
	JoystickInit();
	FlashInit();

#if defined USE_DIRECT_X
//#if defined USE_BITMAPS
	DWORD tick;
	DDBLTFX     ddbltfx;

	srand(GetTickCount() % 0xFFFF);

	SetupGlobal();
	if(DoYesNoMenu("Welcome to Sintendo",
		"I hope you enjoy your stay",
		"Please fasten your seatbelts")) {

		S9xMessage(0, 0, "You selected YES");

	} else {
		S9xMessage(0, 0, "You selected NO");
	}

#if defined UNDER_CE
	splash = LoadBitmapFromFile("\\CD-ROM\\splash.bmp");
#else
	splash = LoadBitmapFromFile("splash.bmp");
#endif
	gdds_back->Blt(NULL, splash, NULL, DDBLT_WAIT, NULL);

	gdds_screen->Flip(NULL, DDFLIP_WAIT);

	tick = GetTickCount();							// Set the timer
	while(GetTickCount() < tick + SPLASH_WAIT);		// Wait
//#endif
#endif

#if defined DO_SNES9X
	DoSnes9xInit();
#endif

	DoSaveLoadMenu(true);
	DoMenu();

#if defined DO_SNES9X
	DoSnes9xEmulation();
#endif
}

////////////////////////////////////////////////////////////////////
// DoMenu
////////////////////////////////////////////////////////////////////
bool DoMenu(bool allowcancel) {
#if defined USE_DIRECT_X
	// Memory allocated, but remains static 
	static HFONT font = NULL;
	LOGFONT lf;
	if(!font) {
		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = FONT_SIZE;
		lf.lfWeight = FW_BOLD;
		font = CreateFontIndirect(&lf);
	}

	LPDIRECTDRAWSURFACE4 pdds2, pdds3 = NULL;
    DDSURFACEDESC2 ddsd;
	HRESULT hr;

#if defined UNDER_CE
	const char picres[3][64] = { "\\CD-ROM\\breezesintendo2.bmp",
		"\\CD-ROM\\sintendo3fade.bmp",
		"\\CD-ROM\\sintendo4.bmp" };
#else
	const char picres[3][64] = { "breezesintendo2.bmp", "sintendo3fade.bmp", "sintendo4.bmp" };
#endif

	// Video memory allocated, release upon transfer
	pdds3 = LoadBitmapFromFile(picres[rand() % 3]);
	// Next line commented out temporarily
//	if(pdds3 == NULL) S9xMessage(0, 0, "Error loading bitmap file");
//	S9xMessage(0, 0, "Load bitmap OK");

	/// Video memory allocated, but remains static
	static LPDIRECTDRAWSURFACE4 accept = NULL, cancel = NULL, cart = NULL, folder = NULL, 
		arrow = NULL, uparrow = NULL, end = NULL;
	if(!accept) {
		accept = LoadBitmap(IDB_ACCEPT);
		cancel = LoadBitmap(IDB_CANCEL);
		cart = LoadBitmap(IDB_CART);
		folder = LoadBitmap(IDB_FOLDER);
		arrow = LoadBitmap(IDB_ARROW);
		uparrow = LoadBitmap(IDB_UPARROW);
		end = LoadBitmap(IDB_END);
	}

#endif

	int start = 0;
	int select = 0;
	int movedlast = -5;

    DDBLTFX					ddbltfx;
	DWORD					tick;

	TCHAR* filenames[128];

#if defined UNDER_CE
	HANDLE hDir;
	hDir = CreateFile(TEXT("\\CD-ROM\\SNES"),
                   GENERIC_READ,
                   FILE_SHARE_READ,
                   NULL,
                   OPEN_EXISTING,
                   FILE_FLAG_BACKUP_SEMANTICS,
                   NULL);

//	char str2[256];
//	sprintf(str2, "SNES Directory handle: %i", hDir);
//	S9xMessage(0, 0, str2);
#endif
 
	// Get file information
	WIN32_FIND_DATA fd;
	HANDLE hand;
	BOOL ok = TRUE;
	int n = 0;

#if defined UNDER_CE
	const TCHAR search[] = TEXT("\\CD-ROM\\SNES\\*.smc");
#else
	const TCHAR search[] = TEXT("SNES\\*.smc");
#endif

	hand = FindFirstFile(search, &fd);
	TCHAR* fn = new TCHAR[_tcslen(fd.cFileName) + 1];
	_tcscpy(fn, fd.cFileName);

	*(_tcschr(fn, '.')) = 0;

	filenames[n] = fn;
	n++;
	
	while(ok && n <= 128) {
		ok = FindNextFile(hand, &fd);
		if(!ok) break;
		fn = new TCHAR[_tcslen(fd.cFileName) + 1];

		_tcscpy(fn, fd.cFileName);

		*(_tcschr(fn, '.')) = 0;
		filenames[n] = fn;
		n++;
	}

#if defined USE_DIRECT_X

	memset(&ddsd, 0, sizeof(ddsd));
    gdds_screen->GetSurfaceDesc(&ddsd);  

    ddsd.dwSize         = sizeof(DDSURFACEDESC2);
    ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;// | DDSCAPS_3DDEVICE;
    ddsd.ddsCaps.dwCaps = DDSCAPS_VIDEOMEMORY;

    ddsd.dwWidth        = 640;
    ddsd.dwHeight       = 480;
	// Video memory allocated, release upon return
    hr = gpdd_directdraw->CreateSurface(&ddsd, &pdds2, NULL);
	if(hr != DD_OK) {
		char str[256];
		sprintf(str, "Error creating DirectDraw surface. %s", ErrorMsg(hr));
		S9xMessage(0, 0, str);
	}
//	S9xMessage(0, 0, "DirectDraw surface created OK");
	if(pdds3 != NULL) {
		pdds2->Blt(NULL, pdds3, NULL, DDBLT_WAIT, NULL);
		// Memory released
		pdds3->Release();
	} else {
		pdds2->Blt(NULL, splash, NULL, DDBLT_WAIT, NULL);
	}
//	S9xMessage(0, 0, "Bitmap Moved");

	if(!allowcancel) {
		for(int i = 240; i >= -4;) {
			RECT srt = { 0, 240-i, 640, 240 };
			RECT drt = { 0, 0, 640, i };
			gdds_back->Blt(&drt, splash, &srt, DDBLT_WAIT, NULL);

			RECT srt2 = { 0, 240, 640, 240+i };
			RECT drt2 = { 0, 480-i, 640, 480 };
			gdds_back->Blt(&drt2, splash, &srt2, DDBLT_WAIT, NULL);

			RECT drt3 = { 0, i, 640, 480-i };
			RECT srt3 = { 0, i, 640, 480-i };
			gdds_back->Blt(&drt3, pdds2, &srt3, DDBLT_WAIT, NULL);
			gdds_screen->Flip(NULL, DDFLIP_WAIT);

			i-= 4;
		}
	}

	gdds_back->Blt(NULL, pdds2, NULL, DDBLT_WAIT, NULL);
	gdds_screen->Flip(NULL, DDFLIP_WAIT);

	// Main menu loop
	while(1) {
		// Draw the background, to be removed later
		gdds_back->Blt(NULL, pdds2, NULL, DDBLT_WAIT, NULL);

		// Direct3D calls
		gdev_device->BeginScene();
//		AlphaBlt(device, pdds2, 0, 0, 640, 480, 1.0);
		FillGradientRect(gdev_device, 0,0,640,480,0xFFFFFFFF, D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0.5,0.5,0.5,1),
			D3DRGBA(0,0,0,1));

		// Draw the menu box
		FillGradientRect(gdev_device, MENU_BORDER, MENU_BORDER, SCREENX - MENU_BORDER, SCREENY - MENU_BORDER,
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));

		// Draw the selection band
		FillGradientRect(gdev_device, MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE,
			SCREENX - MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE + FONT_SIZE,
			D3DRGBA(0.0, 0.3, 1.0, 0.8), D3DRGBA(0.0, 0.3, 1.0, 0.8),
			D3DRGBA(0.0, 0.0, 0.3, 0.8), D3DRGBA(0.0, 0.0, 0.3, 0.8));

		gdev_device->EndScene();

		// Draw accept/cancel
		if(allowcancel) {
			int x = SCREENX/2 - 90;
			int y = SCREENY - 32 - 14;
			RECT r1 = { x, y, x + 90, y + 27 };
			gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
			RECT r3 = { 320, y, 320 + 90, y + 27 };
			gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);
		} else {
			int x = 640/2 - 46;
			int y = 480 - 32 - 14;
			RECT r1 = { x, y, x + 90, y + 27 };
			gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
			RECT r3 = { x+90, y, x+90+2, y + 27 };
			gdds_back->Blt(&r3, end, NULL, DDBLT_WAIT, NULL);
		}

		// Draw the arrows
		RECT r = { MENU_BORDER + 245, MENU_BORDER + 326,
			MENU_BORDER + 245 + 22, MENU_BORDER + 326 + 20 };
		if(n-1 >= start+10) {
			gdds_back->Blt(&r, arrow, NULL, DDBLT_WAIT, NULL);
		}
		RECT r2 = { MENU_BORDER + 245, MENU_BORDER + 6, 
			MENU_BORDER + 245 + 22, MENU_BORDER + 6 + 20 };
		if(start != 0) {
			gdds_back->Blt(&r2, uparrow, NULL, DDBLT_WAIT, NULL);
		}

		// Draw the icon
		RECT rect = { MENU_BORDER + LEFTOF_ICON, MENU_BORDER + FONT_SIZE + 1,
			MENU_BORDER + LEFTOF_ICON + ICONX, MENU_BORDER + FONT_SIZE + ICONY };
		for(int i = 0; i < 9; i++) {
			gdds_back->Blt(&rect, cart, NULL, DDBLT_WAIT, NULL);
			rect.top += FONT_SIZE;
			rect.bottom += FONT_SIZE;
		} 

		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);

		SelectObject(dc, font);

		SetBkMode(dc, TRANSPARENT);

		if(start == 0) {
			SIZE sz;
			SetTextColor(dc, RGB(0, 0, 0));
			GetTextExtentPoint32(dc, TEXT("Sintendo Build #209"), 19, &sz);
			int x = SCREENX / 2 - sz.cx / 2;
			ExtTextOut(dc, x, MENU_BORDER, NULL, NULL, TEXT("Sintendo Build #209"), 19, NULL);
		}

		SetTextColor(dc, RGB(255, 255, 255));
		int y = FONT_SIZE;
		for(int j = start; j < start + 9; j++) {
			if(filenames[j] != NULL)
			ExtTextOut(dc, MENU_BORDER + LEFTOF_ICON + ICONX + LEFTOF_TEXT, MENU_BORDER + y, NULL, NULL, filenames[j], _tcslen(filenames[j]), NULL);
			y += FONT_SIZE;
		}
		
		gdds_back->ReleaseDC(dc);

		// Show it all on the screen
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

		// Accept input
		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					select--;
					if(select < 0) { select = 0; start--; }
					if(start < 0) { start = 0; }
					break;
				}
				if((a | DOWN) == a) {
					select++;
					if(select > 8) { start++; select = 8; }
					if(start + 9 >= n) { start--; }
					if(select >= n) select = n - 1;
					break;
				}
				if((a | BUTTON[3]) == a) {
					select--;
					if(select < 0) { select = 0; start--; }
					if(start < 0) { start = 0; }
					break;
				}
				if((a | BUTTON[4]) == a) {
					select++;
					if(select > 8) { start++; select = 8; }
					if(start + 9 >= n) { start--; }
					if(select >= n) select = n - 1;
					break;
				}
				if((a | BUTTON[1]) == a) {
					goto end;		// ACK
				}
				if(((a | BUTTON[0]) == a) && allowcancel) {
					for(int r = 0; r < n; r++) {
						delete filenames[r];
					}
					CloseHandle(hand);
					pdds2->Release();
					return false;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		}
	}
#endif

end:
	char str[256];
#if !defined UNDER_CE
	strcpy(str, "SNES\\");
#else
	strcpy(str, "\\CD-ROM\\SNES\\");
#endif
	char f[256];
#if defined UNDER_CE
	wcstombs(f, filenames[select+start], 255);
#else
	strcpy(f, filenames[select+start]);
#endif
	strcat(str, f);
	strcat(str, ".smc");

	Memory.LoadROM(str);

	char *fname = (char *)filenames[select+start];
	char SRAMFile[13];
	char SRAMFile2[13];

	int j = 0;
	for(int i = 0; i < 7; i++) {
		SRAMFile[i] = fname[j];
		SRAMFile2[i] = fname[j+1];
		j += 2;
	}

// THis is here because I didn't feel like really solving the problem :)
#if defined UNDER_CE
	j = wcslen(filenames[select+start]) * 2;
#else
	j = strlen(filenames[select+start]);
#endif
	j -= 10;
	for(i = 7; i < 12; i++) {
		SRAMFile[i] = fname[j];
		SRAMFile2[i] = fname[j+1];
		j += 2;
	}

	SRAMFile[12] = 0;
	SRAMFile2[12] = 0;

	for(i = 0; i < 12; i++) {
		if(SRAMFile[i] == ' ') SRAMFile[i] = '_';
	}
	for(i = 0; i < 12; i++) {
		if(SRAMFile2[i] == ' ') SRAMFile[i] = '_';
	}

	char s[256];

	S9xMessage(0, 0, "Begin Load VMS File...");

	if(!Memory.LoadSRAM(SRAMFile)) {
		sprintf(s, "This game requires %i Blocks.", GetSRAMSize());
		if(DoYesNoMenu("No save data found.", s, "Do you wish to enable saving?", NO)) {
			FlashCreateFile(SRAMFile, SRAMFile, Memory.SRAMSize);
		}
	}

	sprintf(str, "Memory.ROMName = %s", Memory.ROMName);
	S9xMessage(0, 0, str);
	sprintf(str, "Memory.ROMFilename = %s", Memory.ROMFilename);
	S9xMessage(0, 0, str);
	sprintf(str, "Memory.ROMId = %s", Memory.ROMId);
	S9xMessage(0, 0, str);
	sprintf(str, "SRAMFile = %s", SRAMFile);
	S9xMessage(0, 0, str);
	sprintf(str, "SRAMFile2 = %s", SRAMFile2);
	S9xMessage(0, 0, str);

	for(int r = 0; r < n; r++) {
		delete filenames[r];
	}
	CloseHandle(hand);
	pdds2->Release();

	return true;
}

////////////////////////////////////////////////////////////////////
// DoInGameMenu
////////////////////////////////////////////////////////////////////
void DoInGameMenu(void) {

	Memory.SaveSRAM("test.srm");

#if defined USE_DIRECT_X
	// Memory allocated, but remains static... perhaps move to global at a later date
	static HFONT font = NULL;
	LOGFONT lf;
	if(!font) {
		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = 32;
		lf.lfWeight = FW_BOLD;
		font = CreateFontIndirect(&lf);
	}

    DDSURFACEDESC2 ddsd;
	HRESULT hr;

	const int nOPTIONS = 7;
	TCHAR options[nOPTIONS][30] = {
		TEXT("Reset SNES"), TEXT("Select Game"), TEXT("Controller Configuration"),
		TEXT("Video Mode"), TEXT("Frameskip"), TEXT("Credits"), TEXT("Return to Game")
	};

	// Video Memory allocated, but remains static... again, perhaps global
	static LPDIRECTDRAWSURFACE4 accept = NULL, cancel = NULL;
	if(!accept) {
		accept = LoadBitmap(IDB_ACCEPT);
		cancel = LoadBitmap(IDB_CANCEL);
	}
	
	// Video Memory allocated, but remains static... again, perhaps global
	static LPDIRECTDRAWSURFACE4 optionpic[nOPTIONS] = {
		NULL, NULL, NULL, NULL, NULL, NULL };
	if(!optionpic[0]) {
		optionpic[0] = LoadBitmap(IDB_RESET);
		optionpic[1] = LoadBitmap(IDB_CART);
		optionpic[2] = LoadBitmap(IDB_CONTROLLER);
		optionpic[3] = LoadBitmap(IDB_VIDEO);
		optionpic[4] = LoadBitmap(IDB_VIDEO);
		optionpic[5] = LoadBitmap(IDB_VIDEO);
		optionpic[6] = LoadBitmap(IDB_RETURN);
	}

	int select = 0;
	int movedlast = -5;

	// Main menu loop
	while(1) {

		// Direct3D Stuff
		gdev_device->BeginScene();
//		AlphaBlt(device, pdds2, 0, 0, 640, 480, 1.0);
		FillGradientRect(gdev_device, 0,0,640,480,0xFFFFFFFF, D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0.5,0.5,0.5,1),
			D3DRGBA(0,0,0,1));

		// Draw the menu box
		FillGradientRect(gdev_device, MENU_BORDER, MENU_BORDER, SCREENX - MENU_BORDER, SCREENY - MENU_BORDER,
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));

		// Draw the selection band
		FillGradientRect(gdev_device, MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE,
			SCREENX - MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE + FONT_SIZE,
			D3DRGBA(0.0, 0.3, 1.0, 0.8), D3DRGBA(0.0, 0.3, 1.0, 0.8),
			D3DRGBA(0.0, 0.0, 0.3, 0.8), D3DRGBA(0.0, 0.0, 0.3, 0.8));

		gdev_device->EndScene();

		// Draw the icons
		RECT rect = { MENU_BORDER + LEFTOF_ICON, MENU_BORDER + FONT_SIZE + 1,
			MENU_BORDER + LEFTOF_ICON + ICONX, MENU_BORDER + FONT_SIZE + ICONY };
		for(int i = 0; i < nOPTIONS; i++) {
			gdds_back->Blt(&rect, optionpic[i], NULL, DDBLT_WAIT, NULL);
			rect.top += FONT_SIZE;
			rect.bottom += FONT_SIZE;
		} 

		// Draw accept/cancel
		int x = 640/2 - 90;
		int y = 480 - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { 320, y, 320 + 90, y + 27 };
		gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);

		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);

		SelectObject(dc, font);

		SetBkMode(dc, TRANSPARENT);

		SIZE sz;
		SetTextColor(dc, RGB(0, 0, 0));
		GetTextExtentPoint32(dc, TEXT("Game Paused"), 11, &sz);
		x = SCREENX / 2 - sz.cx / 2;
		ExtTextOut(dc, x, MENU_BORDER, NULL, NULL, TEXT("Game Paused"), 11, NULL);

		SetTextColor(dc, RGB(255, 255, 255));
		y = 32;
		for(int j = 0; j < nOPTIONS; j++) {
			if(options[j] != NULL)
			ExtTextOut(dc, 64 + 5 + 38 + 8, 64 + y, NULL, NULL, options[j], _tcslen(options[j]), NULL);
			y += 32;
		}
		
		gdds_back->ReleaseDC(dc); 

		// Draw it all to the screen
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

		// Accept input
		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					select--;
					if(select < 0) { select = 0; }
					break;
				}
				if((a | DOWN) == a) {
					select++;
					if(select >= nOPTIONS) { select = nOPTIONS - 1; }
					break;
				}
				if((a | BUTTON[3]) == a) {
					select--;
					if(select < 0) { select = 0; }
					break;
				}
				if((a | BUTTON[4]) == a) {
					select++;
					if(select >= nOPTIONS) { select = nOPTIONS - 1; }
					break;
				}
				if((a | BUTTON[1]) == a) {
					switch(select) {
					case 0:
						S9xReset();
						return;
					case 1:
						DoMenu(true);
						return;
					case 4:
						DoFrameSkipMenu();
						return;
					case 2:
					case 3:
					case 5:
					case 6:
						return;
					}
					return;
				}
				if((a | BUTTON[0]) == a) {
					return;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		}
	}

#endif
}

////////////////////////////////////////////////////////////////////
// DoSaveLoadMenu
////////////////////////////////////////////////////////////////////
void DoSaveLoadMenu(bool save) {
	bool available[2][4];
	memset(available, 0, sizeof(bool) * 12);

	int select = 0;
	int selx = 0;
	int sely = 0;
	int movedlast = -5;

	GetCurrentFlashPort(&selx, &sely);

    DDBLTFX					ddbltfx;
	DWORD					tick;

	while(1) {

		// Direct3D calls
		gdev_device->BeginScene();
//		AlphaBlt(device, pdds2, 0, 0, 640, 480, 1.0);
		FillGradientRect(gdev_device, 0,0,640,480,0xFFFFFFFF, D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0.5,0.5,0.5,1),
			D3DRGBA(0,0,0,1));

		// Draw the menu box
		FillGradientRect(gdev_device, MENU_BORDER, MENU_BORDER, SCREENX - MENU_BORDER, SCREENY - MENU_BORDER,
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));

		int y = MENU_BORDER + MEMORY_CARD_TOP_BOTTOM;
		int x = MENU_BORDER + MEMORY_CARD_LEFT_RIGHT;

		for(int i = 0; i < 4; i++) {
			for(int j = 0; j < 2; j++) {
				D3DCOLOR d3dc;
				if(GetFlashStatus(i, j) != -1) {
					d3dc = D3DRGBA(0.5, 0.5, 1.0, 1.0);
				} else {
					d3dc = D3DRGBA(0.3, 0.3, 0.5, 1.0);
				}
				FillGradientRect(gdev_device, x, y, x + MEMORY_CARD_WIDTH,
					y + MEMORY_CARD_HEIGHT, d3dc, d3dc, d3dc, d3dc);

				if(i == selx && j == sely) {
					FillGradientRect(gdev_device, x-5, y-5, x + MEMORY_CARD_WIDTH + 5,
						y + MEMORY_CARD_HEIGHT + 5, 
						D3DRGBA(0.0, 0.3, 1.0, 0.8), D3DRGBA(0.0, 0.3, 1.0, 0.8),
						D3DRGBA(0.0, 0.0, 0.3, 0.8), D3DRGBA(0.0, 0.0, 0.3, 0.8));
				}

				y += MEMORY_CARD_HEIGHT + MEMORY_CARD_UP_DOWN_SEPERATOR;
			}
			y = MENU_BORDER + MEMORY_CARD_TOP_BOTTOM;
			x += MEMORY_CARD_WIDTH + MEMORY_CARD_LEFT_RIGHT_SEPERATOR;
		}

		
		gdev_device->EndScene();

		// Draw accept
		x = 640/2 - 46;
		y = 480 - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, gSurfAccept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { x+90, y, x+90+2, y + 27 };
		gdds_back->Blt(&r3, gSurfEnd, NULL, DDBLT_WAIT, NULL);
		
		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);

		SelectObject(dc, ghFontLarge);
		SetBkMode(dc, TRANSPARENT);

		SIZE sz;
		SetTextColor(dc, RGB(0, 0, 0));
		GetTextExtentPoint32(dc, TEXT("Select Memory Card"), 18, &sz);
		x = SCREENX / 2 - sz.cx / 2;
		ExtTextOut(dc, x, MENU_BORDER + 16, NULL, NULL, TEXT("Select Memory Card"), 18, NULL);

		TCHAR str[256];
		x = MENU_BORDER + MEMORY_CARD_LEFT_RIGHT + MEMORY_CARD_WIDTH / 2;
		y = MENU_BORDER + MEMORY_CARD_TOP_BOTTOM + MEMORY_CARD_HEIGHT / 2 - sz.cy;

		SetTextColor(dc, RGB(255, 255, 255));
		int free = 0;
		for(i = 0; i < 4; i++) {
			for(int j = 0; j < 2; j++) {
				free = GetFlashStatus(i, j);
				if(free != -1) {
					wsprintf(str, TEXT("%i"), free);
					GetTextExtentPoint32(dc, str, _tcslen(str), &sz);
					ExtTextOut(dc, x - sz.cx / 2, y, NULL, NULL, str, _tcslen(str), NULL);
					GetTextExtentPoint32(dc, TEXT("Free"), 4, &sz);
					ExtTextOut(dc, x - sz.cx / 2, y + sz.cy, NULL, NULL, TEXT("Free"), 4, NULL);
				}

				y += MEMORY_CARD_HEIGHT + MEMORY_CARD_UP_DOWN_SEPERATOR;
			}
			y = MENU_BORDER + MEMORY_CARD_TOP_BOTTOM + MEMORY_CARD_HEIGHT / 2 - sz.cy;
			x += MEMORY_CARD_WIDTH + MEMORY_CARD_LEFT_RIGHT_SEPERATOR;
		}


/*		SelectObject(dc, ghFontSmall);
		SetTextColor(dc, RGB(255, 255, 255));
		x = 38;
		for(int j = 0; j < 4; j++) {
			if(available[0][i]) 
				SetTextColor(dc, RGB(255, 255, 255));
			else
				SetTextColor(dc, RGB(64, 64, 64));
			TCHAR str[256];
			wsprintf(str, TEXT("File %i"), j);
			ExtTextOut(dc, x, 46, NULL, NULL, str, _tcslen(str), NULL);
			x += 128; 
		} */
		
		gdds_back->ReleaseDC(dc); 

		
		/*
		// Draw the selection band
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 20 + (8 << 5);
		RECT rct;
		rct.left = 0;
		rct.right = 640 - MENU_BORDER *2;
//		select++;
		rct.top = (select + 1) * 32;
		rct.bottom = rct.top + 32;
		pdds->Blt(&rct, NULL, &rct, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);

//		S9xMessage(0, 0, "After clear selection band - Supposed to draw arrows and icons");

		// Draw the icon
		RECT rect = { 5, 33, 42, 62 };
		for(int i = 0; i < nOPTIONS; i++) {
			pdds->Blt(&rect, optionpic[i], NULL, DDBLT_WAIT, NULL);
			rect.top += 32;
			rect.bottom += 32;
		} 

//		S9xMessage(0, 0, "After draw arrows and icons - Blt to back buffer"); */
// Blt to the back buffer
//		RECT rt = { MENU_BORDER, MENU_BORDER, 640 - MENU_BORDER, 480 - MENU_BORDER };
//		gdds_back->Blt(&rt, pdds, NULL, DDBLT_WAIT, NULL); 

/*		int x = 640/2 - 90;
		int y = 480 - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { 320, y, 320 + 90, y + 27 };
		gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);


//		S9xMessage(0, 0, "After blt to back buffer - Supposed to draw text");


//		S9xMessage(0, 0, "After draw text - Blt");*/

//		gdds_back->Blt(&rect, cart, NULL, DDBLT_WAIT, NULL); 
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					sely--;
					if(sely < 0) sely = 0;
					break;
				}
				if((a | DOWN) == a) {
					sely++;
					if(sely > 1) sely = 1;
					break;
				}
				if((a | LEFT) == a) {
					selx--;
					if(selx < 0) selx = 0;
					break;
				}
				if((a | RIGHT) == a) {
					selx++;
					if(selx > 3) selx = 3;
					break;
				}
				if((a | BUTTON[0]) == a) {
					return;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		} 

//		gdds_back->Blt(NULL, gdds_screen, NULL, DDBLT_WAIT, NULL);
	} 
//	}

	MSG msg;
	while(1) {
		if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
			if(!GetMessage(&msg, NULL, 0, 0))
				return;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	} 

}

////////////////////////////////////////////////////////////////////
// DoFrameSkipMenu
////////////////////////////////////////////////////////////////////
void DoFrameSkipMenu() {
/*
#if defined USE_DIRECT_X
	// Memory allocated, but remains static 
	static HFONT font = NULL;
	LOGFONT lf;
	if(!font) {
		memset(&lf, 0, sizeof(lf));
		lf.lfHeight = FONT_SIZE;
		lf.lfWeight = FW_BOLD;
		font = CreateFontIndirect(&lf);
	}

	static LPDIRECTDRAWSURFACE4 accept = NULL, cancel = NULL;
	if(!accept) {
		accept = LoadBitmap(IDB_ACCEPT);
		cancel = LoadBitmap(IDB_CANCEL);
	}
	
	const int nSkipOption = 12;
	const int skipoption1[nSkipOption] = { 0, 1, 1, 1, 1, 3, 1, 5, 2, 3, 5, 7 };
	const int skipoption2[nSkipOption] = { 0, 8, 6, 4, 3, 8, 2, 8, 3, 4, 6, 8 };
	
	int start = 0;
	int select = skipmenu;
	int movedlast = -5;
	int n = nSkipOption;

//	gdds_back->Blt(NULL, pdds2, NULL, DDBLT_WAIT, NULL);
//	gdds_screen->Flip(NULL, DDFLIP_WAIT);

	// Main menu loop
	while(1) {
		// Draw the background, to be removed later
//		gdds_back->Blt(NULL, pdds2, NULL, DDBLT_WAIT, NULL);

		// Direct3D calls
		gdev_device->BeginScene();
//		AlphaBlt(device, pdds2, 0, 0, 640, 480, 1.0);
		FillGradientRect(gdev_device, 0,0,640,480,0xFFFFFFFF, D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0.5,0.5,0.5,1),
			D3DRGBA(0,0,0,1));

		// Draw the menu box
		FillGradientRect(gdev_device, MENU_BORDER, MENU_BORDER, SCREENX - MENU_BORDER, SCREENY - MENU_BORDER,
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));

		// Draw the selection band
		FillGradientRect(gdev_device, MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE,
			SCREENX - MENU_BORDER, MENU_BORDER + (select + 1) * FONT_SIZE + FONT_SIZE,
			D3DRGBA(0.0, 0.3, 1.0, 0.8), D3DRGBA(0.0, 0.3, 1.0, 0.8),
			D3DRGBA(0.0, 0.0, 0.3, 0.8), D3DRGBA(0.0, 0.0, 0.3, 0.8));

		gdev_device->EndScene();

		// Draw accept/cancel
		int x = SCREENX/2 - 90;
		int y = SCREENY - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { 320, y, 320 + 90, y + 27 };
		gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);

		// Draw the arrows
		RECT r = { MENU_BORDER + 245, MENU_BORDER + 326,
			MENU_BORDER + 245 + 22, MENU_BORDER + 326 + 20 };
		if(n-1 >= start+10) {
			gdds_back->Blt(&r, arrow, NULL, DDBLT_WAIT, NULL);
		}
		RECT r2 = { MENU_BORDER + 245, MENU_BORDER + 6, 
			MENU_BORDER + 245 + 22, MENU_BORDER + 6 + 20 };
		if(start != 0) {
			gdds_back->Blt(&r2, uparrow, NULL, DDBLT_WAIT, NULL);
		}

		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);

		SelectObject(dc, font);

		SetBkMode(dc, TRANSPARENT);

		if(start == 0) {
			SIZE sz;
			SetTextColor(dc, RGB(0, 0, 0));
			GetTextExtentPoint32(dc, TEXT("Frameskip Options"), 17, &sz);
			int x = SCREENX / 2 - sz.cx / 2;
			ExtTextOut(dc, x, MENU_BORDER, NULL, NULL, TEXT("Frameskip Options"), 17, NULL);
		}

		SetTextColor(dc, RGB(255, 255, 255));
		int y = FONT_SIZE;
		for(int j = start; j < start + 9; j++) {
			TCHAR str[256];
			_stprintf(str, TEXT("Skip %i %s for Every %i %s"), skipoption1[j],
				skipoption1[j] == 1 ? TEXT("Frame") : TEXT("Frames"),
				skipoption2[j],
				skipoption2[j] == 1 ? TEXT("Frame") : TEXT("Frames"))
			ExtTextOut(dc, MENU_BORDER + LEFTOF_ICON + ICONX + LEFTOF_TEXT, MENU_BORDER + y, NULL, NULL, str, _tcslen(filenames[j]), NULL);
			y += FONT_SIZE;
		}
		
		gdds_back->ReleaseDC(dc);

		// Show it all on the screen
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

		// Accept input
		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					select--;
					if(select < 0) { select = 0; start--; }
					if(start < 0) { start = 0; }
					break;
				}
				if((a | DOWN) == a) {
					select++;
					if(select > 8) { start++; select = 8; }
					if(start + 9 >= n) { start--; }
					if(select >= n) select = n - 1;
					break;
				}
				if((a | BUTTON[3]) == a) {
					select--;
					if(select < 0) { select = 0; start--; }
					if(start < 0) { start = 0; }
					break;
				}
				if((a | BUTTON[4]) == a) {
					select++;
					if(select > 8) { start++; select = 8; }
					if(start + 9 >= n) { start--; }
					if(select >= n) select = n - 1;
					break;
				}
				if((a | BUTTON[1]) == a) {
					goto end;		// ACK
				}
				if((a | BUTTON[0]) == a)) {
					for(int r = 0; r < n; r++) {
						delete filenames[r];
					}
					CloseHandle(hand);
					pdds2->Release();
					return false;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		}
	}
#endif

end:
	return;
*/
}

const int YES_NO_MENU_TOP = (SCREENY / 2) - (FONT_SIZE * 3);
const int YES_NO_MENU_BOTTOM = (SCREENY / 2) + (FONT_SIZE * 3);
const int SCREENX_MIDDLE = (SCREENX / 2);
const int SCREENY_MIDDLE = (SCREENY / 2);

bool DoYesNoMenu(const char *line1, const char *line2, const char *line3, int start) {

	int i, x, y;

	// Convert Strings to UNICODE
	TCHAR l[6][256];

	#if defined UNDER_CE
		mbstowcs(l[0], line1, strlen(line1));
		mbstowcs(l[1], line2, strlen(line2));
		mbstowcs(l[2], line2, strlen(line3));
	#else
		strcpy(l[0], line1);
		strcpy(l[1], line2);
		strcpy(l[2], line3);
	#endif

	_tcscpy(l[3], _T(""));
	_tcscpy(l[4], _T("YES"));
	_tcscpy(l[5], _T("NO"));
	
	// Get the size of the largest string
	HDC	dc;
	gdds_back->GetDC(&dc);

	SelectObject(dc, ghFontLarge);

	int max = 0;
	SIZE sz;

	for(i = 0; i < 3; i++) {
		GetTextExtentPoint32(dc, l[i], _tcslen(l[i]), &sz);
		if(sz.cx > max) max = sz.cx;
	}

	gdds_back->ReleaseDC(dc);

	int selection = 1 - (int)start;

	// Do the menu loop
	while(1) {
		// Direct3D calls
		gdev_device->BeginScene();
//		AlphaBlt(device, pdds2, 0, 0, 640, 480, 1.0);
		FillGradientRect(gdev_device, 0,0,640,480,0xFFFFFFFF, D3DRGBA(0.5,0.5,0.5,1), D3DRGBA(0.5,0.5,0.5,1),
			D3DRGBA(0,0,0,1));

		// Draw the menu box
		FillGradientRect(gdev_device, 
			SCREENX_MIDDLE - (max / 2) - 16,
			YES_NO_MENU_TOP,
			SCREENX_MIDDLE + (max / 2) + 16,
			YES_NO_MENU_BOTTOM,
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5), 
			D3DRGBA(0.0, 0.2, 1.0, 0.5), D3DRGBA(0.0, 0.2, 1.0, 0.5));

		// Draw the selection band
		FillGradientRect(gdev_device, 
			SCREENX_MIDDLE - (max / 2) - 16,
			SCREENY_MIDDLE + FONT_SIZE + (FONT_SIZE * selection),
			SCREENX_MIDDLE + (max / 2) + 16,
			SCREENY_MIDDLE + (FONT_SIZE * 2) + (FONT_SIZE * selection),
			D3DRGBA(0.0, 0.3, 1.0, 0.8), D3DRGBA(0.0, 0.3, 1.0, 0.8),
			D3DRGBA(0.0, 0.0, 0.3, 0.8), D3DRGBA(0.0, 0.0, 0.3, 0.8));

		gdev_device->EndScene();

		// Draw accept
		x = 640/2 - 46;
		y = 480 - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, gSurfAccept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { x+90, y, x+90+2, y + 27 };
		gdds_back->Blt(&r3, gSurfEnd, NULL, DDBLT_WAIT, NULL);
		
		// Draw the text
		HDC	dc;
		gdds_back->GetDC(&dc);

		SelectObject(dc, ghFontLarge);
		SetBkMode(dc, TRANSPARENT);

		SIZE sz;
		SetTextColor(dc, RGB(255, 255, 255));
		for(i = 0; i < 6; i++) {
			GetTextExtentPoint32(dc, l[i], _tcslen(l[i]), &sz);
			ExtTextOut(dc, 
				SCREENX_MIDDLE - (sz.cx / 2),
				YES_NO_MENU_TOP + (FONT_SIZE * i),
				NULL, NULL, l[i], _tcslen(l[i]), NULL);
		}

		TCHAR str[256];
		x = MENU_BORDER + MEMORY_CARD_LEFT_RIGHT + MEMORY_CARD_WIDTH / 2;
		y = MENU_BORDER + MEMORY_CARD_TOP_BOTTOM + MEMORY_CARD_HEIGHT / 2 - sz.cy;



/*		SelectObject(dc, ghFontSmall);
		SetTextColor(dc, RGB(255, 255, 255));
		x = 38;
		for(int j = 0; j < 4; j++) {
			if(available[0][i]) 
				SetTextColor(dc, RGB(255, 255, 255));
			else
				SetTextColor(dc, RGB(64, 64, 64));
			TCHAR str[256];
			wsprintf(str, TEXT("File %i"), j);
			ExtTextOut(dc, x, 46, NULL, NULL, str, _tcslen(str), NULL);
			x += 128; 
		} */
		
		gdds_back->ReleaseDC(dc); 

		
		/*
		// Draw the selection band
		memset(&ddbltfx, 0, sizeof(DDBLTFX));
		ddbltfx.dwSize = sizeof(DDBLTFX);
		ddbltfx.dwFillColor = 20 + (8 << 5);
		RECT rct;
		rct.left = 0;
		rct.right = 640 - MENU_BORDER *2;
//		select++;
		rct.top = (select + 1) * 32;
		rct.bottom = rct.top + 32;
		pdds->Blt(&rct, NULL, &rct, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx);

//		S9xMessage(0, 0, "After clear selection band - Supposed to draw arrows and icons");

		// Draw the icon
		RECT rect = { 5, 33, 42, 62 };
		for(int i = 0; i < nOPTIONS; i++) {
			pdds->Blt(&rect, optionpic[i], NULL, DDBLT_WAIT, NULL);
			rect.top += 32;
			rect.bottom += 32;
		} 

//		S9xMessage(0, 0, "After draw arrows and icons - Blt to back buffer"); */
// Blt to the back buffer
//		RECT rt = { MENU_BORDER, MENU_BORDER, 640 - MENU_BORDER, 480 - MENU_BORDER };
//		gdds_back->Blt(&rt, pdds, NULL, DDBLT_WAIT, NULL); 

/*		int x = 640/2 - 90;
		int y = 480 - 32 - 14;
		RECT r1 = { x, y, x + 90, y + 27 };
		gdds_back->Blt(&r1, accept, NULL, DDBLT_WAIT, NULL);
		RECT r3 = { 320, y, 320 + 90, y + 27 };
		gdds_back->Blt(&r3, cancel, NULL, DDBLT_WAIT, NULL);


//		S9xMessage(0, 0, "After blt to back buffer - Supposed to draw text");


//		S9xMessage(0, 0, "After draw text - Blt");*/

//		gdds_back->Blt(&rect, cart, NULL, DDBLT_WAIT, NULL); 
		gdds_screen->Flip(NULL, DDFLIP_WAIT);

		int movedlast = 0;
		if(!movedlast) {
			for(int d = 0; d < 4; d++) {
				int a = GetInputDeviceData(d);
				if((a | UP) == a) {
					selection--;
					if(selection < 0) selection = 0;
					break;
				}
				if((a | DOWN) == a) {
					selection++;
					if(selection > 1) selection = 1;
					break;
				}
				if((a | BUTTON[0]) == a) {
					return selection = 0 ? NO : YES;
				}
				if(a) break;
			}
			movedlast++;
		} else {
			if(movedlast < 1) movedlast++;
			else movedlast = 0;
		} 
	}
}