#include "snes9x/snes9x.h"
#include "bitmap.h"
#include "global.h"
#include "snes9x/port.h"

#define TRACE_BITMAP

inline WORD Convert24BitTo16Bit(BYTE red, BYTE green, BYTE blue) {
	WORD w;
#if defined UNDER_CE
	w = ((red >> 3) << 11) |
		((green >> 2) << 5) |
		(blue >> 3);
#else
	w = ((red >> 3) << 10) |
		((green >> 3) << 5) |
		(blue >> 3);
#endif
	return w;
}



LPDIRECTDRAWSURFACE4 LoadBitmapFromFile(const char* file) {

	return NULL;

	STREAM					bmpfile;
	BITMAPFILEHEADER		bfheader;
	BITMAPINFOHEADER		bmpinfo;
	LPVOID					bits;

    DDSURFACEDESC2			ddsd;       // Surface Descriptor
    LPDIRECTDRAWSURFACE4	pdds;       // Pointer to a DirectDraw surface
	HRESULT					hr;

	char str[256];

	bmpfile = OPEN_STREAM(file, "rb");
	
	if(bmpfile == NULL) {
		S9xMessage(0, 0, "Error loading Bitmap File: File not Found");
		return NULL;
	}
//	char fstr[256];
//	sprintf(fstr, "%s.trace.txt", file);

//	FILE *f = fopen(fstr, "wt");

	// Read the file header
	int n = READ_STREAM(&bfheader, sizeof(BITMAPFILEHEADER), bmpfile);
	if(n != sizeof(BITMAPFILEHEADER)) {
		S9xMessage(0, 0, "Header not read properly");
		CLOSE_STREAM(bmpfile);
		return NULL;
	}

#if defined TRACE_BITMAP
//	fprintf(str, "Read File Header\n  Size = %i, Type = %X, Offset = %i\n",
//		bfheader.bfSize, bfheader.bfType, bfheader.bfOffBits);
//	S9xMessage(0, 0, str);
#endif
	
	// Check it for validity
	if(bfheader.bfType != 0x4d42) {
		S9xMessage(0, 0, "Invalid Bitmap");
		CLOSE_STREAM(bmpfile);
		return NULL;
	}

	// Read the bitmap info header, assume NO palette
	n = READ_STREAM(&bmpinfo, sizeof(BITMAPINFOHEADER), bmpfile);
	if(n != sizeof(BITMAPINFOHEADER)) {
		S9xMessage(0, 0, "Corrupt Bitmap");
		CLOSE_STREAM(bmpfile);
		return NULL;
	}

#if defined TRACE_BITMAP
	sprintf(str, "Read info Header\n  %i x %i x %i Size = %i\n",
		bmpinfo.biWidth, bmpinfo.biHeight, bmpinfo.biBitCount,
		bmpinfo.biSizeImage);
	S9xMessage(0, 0, str);
#endif

	if(bmpinfo.biBitCount != 24) {
		S9xMessage(0, 0, "Invalid Bit Depth");
		CLOSE_STREAM(bmpfile);
		return NULL;
	}


	bits = new char[bmpinfo.biHeight * bmpinfo.biWidth * 3];
	if(bits == NULL) {
		S9xMessage(0, 0, "Error allocating bitmap memory");
		CLOSE_STREAM(bmpfile);
		return NULL;
	}

	n = READ_STREAM(bits, bmpinfo.biHeight * bmpinfo.biWidth * 3, bmpfile);
#if defined TRACE_BITMAP
	sprintf(str, "Read %i Bits\n  n= %i", bmpinfo.biHeight * bmpinfo.biWidth * 3, n);
	S9xMessage(0, 0, str);
#endif
//	fclose(f);
//	f = fopen(fstr, "at");
	if(n != bmpinfo.biHeight * bmpinfo.biWidth * 3) {
		S9xMessage(0, 0, "Corrupt Bitmap, not enough bitmap info");
		CLOSE_STREAM(bmpfile);
		return NULL;
	}

	// Create a DirectDrawSurface for this texture
    memset(&ddsd, 0, sizeof(ddsd));
    gdds_screen->GetSurfaceDesc(&ddsd);  

    ddsd.dwSize         = sizeof(DDSURFACEDESC2);
    ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_VIDEOMEMORY;

    ddsd.dwWidth        = bmpinfo.biWidth;
    ddsd.dwHeight       = bmpinfo.biHeight;
    hr = gpdd_directdraw->CreateSurface(&ddsd, &pdds, NULL);
	if(hr != DD_OK) {
//		fprintf(f, "Failed to Create DirectDraw surface");
//		fclose(f);
		S9xMessage(0, 0, "Failed to create DirectDraw Surface");
        return NULL;
	}

    memset(&ddsd, 0, sizeof(ddsd));
    ddsd.dwSize = sizeof(DDSURFACEDESC2);

	hr = pdds->Lock(NULL, &ddsd, DDLOCK_WAIT, NULL);	
	if(hr != DD_OK) { 
//		fprintf(f, "Failed to lock DirectDraw surface");
//		fclose(0);
		S9xMessage(0, 0, "Failed to lock DirectDraw Surface");
		return NULL; 
	}

#if defined TRACE_BITMAP
	sprintf(str, "Bitmap %i x %i, Size %i\n", bmpinfo.biWidth, bmpinfo.biHeight,
		bmpinfo.biHeight * bmpinfo.biWidth * 3);
	S9xMessage(0, 0, str);
#endif

//	fclose(f);
//	f = fopen(fstr, "at");

	WORD *sp = (WORD *)ddsd.lpSurface;
	BYTE *bp = (BYTE *)bits;
	bp += bmpinfo.biHeight * bmpinfo.biWidth * 3 - bmpinfo.biWidth * 3;

	for(int y = 0; y < bmpinfo.biHeight; y++) {
//		fprintf(f, "Line %i, BP = %X, SP = %X\n  ddsd.lpSurface = %X, MAX = %X\n  bits = %X, MAX = %X\n",
//			y, bp, sp, ddsd.lpSurface, (char*)ddsd.lpSurface + bmpinfo.biHeight * bmpinfo.biWidth * 2,
//			(char *)bits, (char *)bits + bmpinfo.biHeight * bmpinfo.biWidth * 3);
//		fclose(f);
//		f = fopen(fstr, "at");
		
		for(int x = 0; x < bmpinfo.biWidth; x++) {
			*sp = Convert24BitTo16Bit(*(bp+2), *(bp+1), *(bp));
//			fprintf(f, "    Bit %i: Red %i, Green %i, Blue %i = 16-Bit %X   -   BP = %X\n",
//				x, *(bp+2), *(bp+1), *(bp), *sp, bp);
			sp++; bp += 3;
		}
		bp -= bmpinfo.biWidth * 6;
	}

	pdds->Unlock(NULL);

	CLOSE_STREAM(bmpfile);
	delete bits;

    return pdds;
}
